//el archivo contiene las configuraciones correspondientes a la seleccion
//clustering

package workspace.Clustering;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import workspace.Main_Set.SimpleNode;
import workspace.Prepro.IntervalNode;
import workspace.Prepro.UmbralNode;

public class ClusteringNode extends SimpleNode{
    //se esta forzando a variable
    private Class<?>[] clases;
    private ArrayList<SimpleNode> nodos;
    //las siguientes banderas establecen si el objeto ya fue configurado
    //o ejecutado
    public boolean configured = false;
    public boolean executed   = false;
    public String pathLogServer = "";
    public int tipoL;
    public int tSesion;
    
    public ClusteringNode(Point center) {
        //se establece centro, dimension
        this.setCenter(center);
        this.setDimension(new Dimension(80,60));
        //se valida el nodo antecesor, 2 nodos le pueden anteceder
        this.clases = new Class<?>[2];
        this.clases[0] = IntervalNode.class;
        this.clases[1] = UmbralNode.class;
        //se instancia un ArrayList en nodos
        this.nodos = new ArrayList();
    }
    
    public Class<?>[] getValidInputClasses(){
        //retorna los antecesores... umbral e interval en este caso
        return this.clases;
    }
    
    public ArrayList<SimpleNode> getInputNodes(){
        //se devuelven los valores de los nodos en un array forzado a tipo
        //SimpleNode
        return this.nodos;
    }
    
    public Image getImage(){
        //el icono
        return new ImageIcon(System.class.getResource("/media/clustering.png")).getImage();
    }
    
    public String getString(){
        //retorna una cadena que lo identifica
        return "k-medias";
    }
    
    //si el nodo puede ser conectado, entonces se aade un nodo
    public void connect(SimpleNode n){
        if(this.canConnect(n)){
            this.nodos.add(n);
        }
    }
    
    //al desconectar el nodo se elimina la linea punteada y se coloca el color
    //de borde por defecto
    public void disconnect(SimpleNode n){
        this.nodos.remove(n);
        this.setBorderColor(null);
    }
    
    //retorna verdadero porque puede ser configurado
    public boolean canConfigure(){
        return true;
    }
    
    //llama al formulario de configuracion de Clustering
    public void configure() {
          this.setConfigured(true);
    }
    
    //retorna verdadero porque puede ser ejecutado
    public boolean canExecute() {
        return true;
    }
    
    public void execute() {
        if(this.getInputNodes().size()==1){
            String sourceClassName = this.getInputNodes().get(0).getClass().getCanonicalName();
            if(sourceClassName.equals(IntervalNode.class.getCanonicalName())){
                
                IntervalNode in = (IntervalNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia interval y a su vez del web server*/
                //***********************************************************
                this.setPathLogServer(in.getPathLogServer());
                System.out.println(" From Clustering: " + this.getPathLogServer());
                //paso del tiempo de sesion desde prepro                
                this.setTsesion(Integer.parseInt(in.getTsesion()));
                System.out.println(" From Clustering: session time " + this.getTsesion());
                System.out.println(" From Clustering: is umbral mfr " + in.isMfr());
                
                if(!in.isConfigured()){
                    this.getParent().showError("The Interval Session must be configured first");
                    //validacion para las baderas ejecucion, configuracion y el color del borde
                        this.setExecuted(false);
                        if(this.isConfigured() == false){
                            this.setConfigured(false);
                            setBorderColor(null);
                        }
                        else{
                            this.setConfigured(true);
                            setBorderColor(new Color(0,0,255));
                        }
                } else if(!in.isExecuted()){
                    this.getParent().showError("The Interval Session must be executed first");
                    //validacion para las baderas ejecucion, configuracion y el color del borde
                        this.setExecuted(false);
                        if(this.isConfigured() == false){
                            this.setConfigured(false);
                            setBorderColor(null);
                        }
                        else{
                            this.setConfigured(true);
                            setBorderColor(new Color(0,0,255));
                        }
                } else if(!this.isConfigured()){
                    this.getParent().showError("First configure after execute");
                    //cuando no esta configurado
                        this.setExecuted(false);
                        this.setConfigured(false);
                        setBorderColor(null);
                } else{
                    
                    this.setExecuted(true);
                    //pinta de verde si el nodo ya se ejecuto
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();                                        
                }
            }else if(sourceClassName.equals(UmbralNode.class.getCanonicalName())){
                
                UmbralNode   un = (UmbralNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia umbral y a su vez del web server*/
                //***********************************************************
                this.setPathLogServer(un.getPathLogServer());
                System.out.println(" From Clustering: " + this.getPathLogServer());
                 //paso del tiempo de sesion desde prepro
                 this.setTsesion(Integer.parseInt(un.getTsesion()));
                 System.out.println(" From Clustering: session time " + this.getTsesion());
                 System.out.println(" From Clustering: is umbral mfr " + un.isMfr());
                    
                if(!un.isConfigured()){
                    this.getParent().showError("The Umbral Session must be configured first");
                    //validacion para las baderas ejecucion, configuracion y el color del borde
                        this.setExecuted(false);
                        if(this.isConfigured() == false){
                            this.setConfigured(false);
                            setBorderColor(null);
                        }
                        else{
                            this.setConfigured(true);
                            setBorderColor(new Color(0,0,255));
                        }
                } else if(!un.isExecuted()){
                    this.getParent().showError("The Umbral Session must be executed first");
                    //validacion para las baderas ejecucion, configuracion y el color del borde
                        this.setExecuted(false);
                        if(this.isConfigured() == false){
                            this.setConfigured(false);
                            setBorderColor(null);
                        }
                        else{
                            this.setConfigured(true);
                            setBorderColor(new Color(0,0,255));
                        }
                } else if(!this.isConfigured()){
                    this.getParent().showError("First configure after execute");
                    //cuando no esta configurado
                    this.setExecuted(false);
                    this.setConfigured(false);
                    setBorderColor(null);
                } else{
                    //EXECUTION CODE GOES HERE
                    //cambia el valor de la bandera executed
                    this.setExecuted(true);
                    //si el nodo fue configurado, el borde se torna verde
                    setBorderColor(new Color(0,255,0));
                    //obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    //al mostrarse graficamente presenta pixeles defectuosos
                    getParent().repaint();                 
                }
            }
        }else{
            this.getParent().showError("First add a connection");
            //cuando no esta configurado
            this.setExecuted(false);
            if(this.isConfigured() == false){
                this.setConfigured(false);
                setBorderColor(null);
            } else{
                this.setConfigured(true);
                setBorderColor(new Color(0,0,255));
            }
            getParent().repaint();
        }
    }   
    
    public String getPathLogServer() {
        return pathLogServer;
    }
    
    public void setPathLogServer(String pathLogServer) {
        this.pathLogServer = pathLogServer;
    }
    
    public int getTipoL() {
        return tipoL;
    }
    
    public void setTipoL(int tipoL) {
        this.tipoL = tipoL;
    }

    public int getTsesion() {
        return tSesion;
    }

    public void setTsesion(int tSesion) {
        this.tSesion = tSesion;
    }

    public boolean isConfigured() {
        return configured;
    }

    public void setConfigured(boolean configured) {
        this.configured = configured;
    }
  
}
