/**el archivo contiene las configuraciones correspondientes a la seleccion grammar
*desde View*/

package workspace.Views;

import grammar.Window;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import workspace.Association.algorithms.AprioriNode;
import workspace.Classification.algorithms.C45Node;
import workspace.Classification.algorithms.CBANode;
import workspace.Clustering.ClusteringNode;
import workspace.Association.algorithms.EquipAssoNode;
import workspace.Association.algorithms.FpTreeNode;
import workspace.HPG.HPGNode;
import workspace.Classification.algorithms.MateTreeNode;
import workspace.Main_Set.SimpleNode;
import workspace.Prepro.IntervalNode;

public class GrammarViewNode extends SimpleNode{
    /**se esta forzando a variable*/
    private Class<?>[] clases;
    private ArrayList<SimpleNode> nodos;
    /**las siguientes banderas establecen si el objeto ya fue configurado
    *o ejecutado*/
    public boolean configured = true;    
    public boolean executed = false;    
    /**path desde el select source*/
    public String pathLogServer = "";
    /**tipoL desde el select source*/
    public int tipoL;
    /**tiempo de sesion desde prepro*/
    public int tSesion;
    
    public GrammarViewNode(Point center) {
        /**se establece centro, dimension*/
        this.setCenter(center);
        this.setDimension(new Dimension(80,60));
        /**se valida el nodo antecesor, 2 nodos le pueden anteceder*/
        this.clases = new Class<?>[1];        
        this.clases[0] = HPGNode.class;
        /**se instancia un ArrayList en nodos*/
        this.nodos = new ArrayList();
    }
    
    public Class<?>[] getValidInputClasses(){
        /**retorna los antecesores...*/
        return this.clases;
    }
    
    public ArrayList<SimpleNode> getInputNodes(){
        /**se devuelven los valores de los nodos en un array forzado a tipo
        *SimpleNode*/
        return this.nodos;
    }
    
    public Image getImage(){
        /**el icono*/
        return new ImageIcon(System.class.getResource("/media/grammar.png")).getImage();
    }
    
    public String getString(){
        /**retorna una cadena que lo identifica*/
        return "Grammar";
    }
    
    /**si el nodo puede ser conectado, entonces se aade un nodo*/
    public void connect(SimpleNode n){
        if(this.canConnect(n)){
            this.nodos.add(n);
        }
    }
    
    /**al desconectar el nodo se elimina la linea punteada y se coloca el color
    *de borde por defecto*/
    public void disconnect(SimpleNode n){
        this.nodos.remove(n);
        this.setBorderColor(null);
    }
           
    /**retorna verdadero porque puede ser ejecutado*/
    public boolean canExecute() {
        return true;
    }
    
    public void execute() {
        if(this.getInputNodes().size()==1){
            String sourceClassName = this.getInputNodes().get(0).getClass().getCanonicalName();
             if (sourceClassName.equals(HPGNode.class.getCanonicalName())){
                HPGNode hn = (HPGNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia HPG*/
                //***********************************************************
                this.setPathLogServer(hn.getPathLogServer());
                System.out.println(" From GrammarView: " + this.getPathLogServer());
                /**paso del tiempo de sesion desde prepro*/
                //this.setTsesion(hn.getTsesion());
                //System.out.println(" From GrammarView: session time " + this.getTsesion());
                
                if(!hn.isConfigured()){
                    this.getParent().showError("The HPG must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!hn.isExecuted()){
                    this.getParent().showError("The HPG must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                }   else{                    
                    
                    // *** Polaris *** Johana
                    long time = System.currentTimeMillis();
            //
                    if(hn==null){
                        this.setMsg("Error: HPG Node don't load...");
                    }else if(hn.getHPG()==null){
                        this.setMsg("Error: Productions aren't exist");
                    }else if(hn.getPathLogServer()==null){
                        this.setMsg("Error: Log Path aren't exist");
                    }else{
                    Window wh= new Window(hn.getHPG(), hn.getPathLogServer());
                    // centrar
                    wh.setLocationRelativeTo(wh.getParent());
                    // Visible
                    wh.setVisible(true);                    
                    }
                    //
                    long executionTime = System.currentTimeMillis() - time;                       
                    System.out.println("GrammarView tiempo: " + executionTime);
                    // *** Finished ***                    
                    this.setExecuted(true);
                    /**pinta de verde si el nodo ya se ejecuto*/
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();
                }
            }/**else if*/
        }else{ 
            this.getParent().showError("First add a connection");
             /**validacion para las baderas ejecucion, configuracion y el color del borde*/            
            this.setExecuted(false);
            setBorderColor(null);
            getParent().repaint();
        }
    }/**execution*/
    
    public String getPathLogServer() {
        return pathLogServer;
    }
    
    public void setPathLogServer(String pathLogServer) {
        this.pathLogServer = pathLogServer;
    }
    
    public int getTipoL() {
        return tipoL;
    }
    
    public void setTipoL(int tipoL) {
        this.tipoL = tipoL;
    }       
}
