/**el archivo contiene las configuraciones correspondientes a la seleccion rules
*desde View**/

package workspace.Views;

import Utils.AssocRules;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import workspace.Association.algorithms.AprioriNode;
import workspace.Association.algorithms.EquipAssoNode;
import workspace.Association.algorithms.FpTreeNode;
import workspace.Classification.algorithms.C45Node;
import workspace.Classification.algorithms.MateTreeNode;
import workspace.Main_Set.SimpleNode;

public class RulesViewNode extends SimpleNode{
    /**se esta forzando a variable*/
    private Class<?>[] clases;
    private ArrayList<SimpleNode> nodos;
    /**las siguientes banderas establecen si el objeto ya fue configurado
    *o ejecutado*/
    public boolean configured = true;
    public boolean executed   = false;
    /**path desde el select source*/
    public String pathLogServer = "";
    /**tipoL desde el select source*/
    public int tipoL;
    /**tiempo de sesion desde prepro*/
    public int tSesion;
    
    public RulesViewNode(Point center) {
        /**se establece centro, dimension*/
        this.setCenter(center);
        this.setDimension(new Dimension(80,60));
        /**se valida el nodo antecesor, 2 nodos le pueden anteceder*/
        this.clases = new Class<?>[5];
        this.clases[0] = AprioriNode.class;
        this.clases[1] = EquipAssoNode.class;
        this.clases[2] = FpTreeNode.class;
        this.clases[3] = C45Node.class;
        this.clases[4] = MateTreeNode.class;
        /*se instancia un ArrayList en nodos*/
        this.nodos = new ArrayList();
    }
    
    public Class<?>[] getValidInputClasses(){
        /**retorna los antecesores...*/
        return this.clases;
    }
    
    public ArrayList<SimpleNode> getInputNodes(){
        /**se devuelven los valores de los nodos en un array forzado a tipo
        *SimpleNode*/
        return this.nodos;
    }
    
    public Image getImage(){
        /**el icono*/
        return new ImageIcon(System.class.getResource("/media/rules.png")).getImage();
    }
    
    public String getString(){
        /**retorna una cadena que lo identifica*/
        return "Rules";
    }
    
    /**si el nodo puede ser conectado, entonces se aade un nodo*/
    public void connect(SimpleNode n){
        if(this.canConnect(n)){
            this.nodos.add(n);
        }
    }
    
    /**al desconectar el nodo se elimina la linea punteada y se coloca el color
    *de borde por defecto*/
    public void disconnect(SimpleNode n){
        this.nodos.remove(n);
        this.setBorderColor(null);
    }   
    
    /**retorna verdadero porque puede ser ejecutado*/
    public boolean canExecute() {
        return true;
    }
    
    public void execute() {
     // *** Polaris *** Johana ***
        AssocRules ar;
        // *** Finished ***
        if(this.getInputNodes().size()==1){
            String sourceClassName = this.getInputNodes().get(0).getClass().getCanonicalName();
            if(sourceClassName.equals(AprioriNode.class.getCanonicalName())){
                
                AprioriNode an = (AprioriNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia apriori*/
                //**********************************************************
                /**paso del path desde select source*/
                this.setPathLogServer(an.getPathLogServer());
                System.out.println(" From Rules V: " + this.getPathLogServer());
                
                /**paso del tiempo de sesion desde prepro*/
                /**validaciones correspondientes en caso de que no este validado*/
                if(!an.isConfigured()){
                    this.getParent().showError("Apriori must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!an.isExecuted()){
                    this.getParent().showError("Apriori must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**se visibiliza el formulario*/
                    // *** Polaris *** Johana ***
                    long time = System.currentTimeMillis();
                    // *** Reglas de Asociacion
                    if(an.getArbolFrec()==null){
                        this.setMsg("Error: Frequents Items don't exists");
                    }else if(an.getDataSet()==null){
                        this.setMsg("Error: Pages Names don't exists");
                    }else if(an.getConfidence()<=0){
                        this.setMsg("Error: Confidence Settings don't exists");
                    }else{
                    ar = new AssocRules(an.getArbolFrec(), an.getDataSet().getDictionary(), an.getConfidence());
                    if(an.isLift()){
                        ar.setLift(true);
                    }
                    ar.buildRules();
                    
                    System.out.println("#Reglas: " + ar.getRules().size());
                    this.setMsg("#Reglas: " + ar.getRules().size());
                    MostrarTabla mt= new MostrarTabla(ar,an.getPathLogServer(), Double.toString(an.getSupport()), Double.toString(an.getConfidence()),
                            "Association Rules","Apriori");
                    /** centrar*/
                    mt.setLocationRelativeTo(mt.getParent());
                    /**Visible*/
                    mt.setVisible(true);                    
                    }
                   //
                    long executionTime = System.currentTimeMillis() - time;                       
                    System.out.println("RulesView tiempo: " + executionTime);                                       
                    // *** Finished ***
                    //getParent().repaint();
                    
                    this.setExecuted(true);
                    /**pinta de verde si el nodo ya se ejecuto*/
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();
                }
            }else if(sourceClassName.equals(EquipAssoNode.class.getCanonicalName())){
                
                EquipAssoNode   en = (EquipAssoNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia equipasso*/
                //***********************************************************
                
                this.setPathLogServer(en.getPathLogServer());
                System.out.println(" From Rules V: " + this.getPathLogServer());
                if(!en.isConfigured()){
                    this.getParent().showError("EquipAsso must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!en.isExecuted()){
                    this.getParent().showError("EquipAsso must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**EXECUTION CODE GOES HERE*/
                    //se visibiliza el formulario
                    // *** Polaris *** Johana ***
                    // *** Reglas de Asociacion
                    if(en.getArbolFrec()==null){
                        this.setMsg("Error: Frequents Items don't exists");
                    }else if(en.getDataSet()==null){
                        this.setMsg("Error: Pages Names don't exists");
                    }else if(en.getConfidence()<=0){
                        this.setMsg("Error: Confidence Settings don't exists");
                    }else{
                    ar = new AssocRules(en.getArbolFrec(), en.getDataSet().getDictionary(), en.getConfidence());
                    if(en.isLift()){
                        ar.setLift(true);
                    }
                    ar.buildRules();
                    
                    this.setMsg("#Reglas: " + ar.getRules().size());
                    MostrarTabla ma= new MostrarTabla(ar,en.getPathLogServer(), Double.toString(en.getSupport()), 
                            Double.toString(en.getConfidence()),"Association Rules","EquipAsso");
                    /**centrar*/
                    ma.setLocationRelativeTo(ma.getParent());
                    /**Visible*/
                    ma.setVisible(true);                                    
                    }
                    // *** Finished ***
                    //getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    *al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            } else if(sourceClassName.equals(FpTreeNode.class.getCanonicalName())){
                
                FpTreeNode   fn = (FpTreeNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia fptree*/
                //***********************************************************
                
                this.setPathLogServer(fn.getPathLogServer());
                System.out.println(" From Rules V: " + this.getPathLogServer());
                if(!fn.isConfigured()){
                    this.getParent().showError("FpTree must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!fn.isExecuted()){
                    this.getParent().showError("FpTree must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**EXECUTION CODE GOES HERE*/
                    /**se visibiliza el formulario*/
                    // *** Polaris *** Johana ***
                    // *** Reglas de Asociacion
                    if(fn.getArbolFrec()==null){
                        this.setMsg("Error: Frequents Items don't exists");
                    }else if(fn.getDataSet()==null){
                        this.setMsg("Error: Pages Names don't exists");
                    }else if(fn.getConfidence()<=0){
                        this.setMsg("Error: Confidence Settings don't exists");
                    }else{
                    ar = new AssocRules(fn.getArbolFrec(), fn.getDataSet().getDictionary(), fn.getConfidence());
                    if(fn.isLift()){
                        ar.setLift(true);
                    }
                    ar.buildRules();
                    
                    this.setMsg("#Reglas: " + ar.getRules().size());
                    MostrarTabla ma= new MostrarTabla(ar,fn.getPathLogServer(), Double.toString(fn.getSupport()), Double.toString(fn.getConfidence()),
                            "Association Rules","FpGrowth");
                    /**centrar*/
                    ma.setLocationRelativeTo(ma.getParent());
                    /**Visible*/
                    ma.setVisible(true);                                       
                    }
                    // *** Finished ***
                    //getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }                                     
             else if(sourceClassName.equals(C45Node.class.getCanonicalName())){
                
                C45Node   an = (C45Node)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia fptree*/
                //***********************************************************
                
                this.setPathLogServer(an.getPathLogServer());
                System.out.println(" From Rules V: " + this.getPathLogServer());
                if(!an.isConfigured()){
                    this.getParent().showError("C4.5 must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!an.isExecuted()){
                    this.getParent().showError("C4.5 must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**EXECUTION CODE GOES HERE*/
                    /**se visibiliza el formulario*/
                    // *** Polaris *** Johana ***
                    // *** Reglas de Asociacion
                    //
                    if(an==null){
                        this.setMsg("Error: C4.5 Node don't load...");
                    }else if(an.getArbol()==null){
                        this.setMsg("Error: Desicion Tree isn't exist");
                    }else if(an.getPathLogServer()==null || an.getTarget()==null){
                        this.setMsg("Error: Settings Parameters don't exist");
                    }else{                        
                        createRulesCla crc= new createRulesCla(an.getArbol(), an.getTarget());
                        //crc.MostrarReglas();
                        MostrarTabla mt= new MostrarTabla(crc.getreglas(),an.getPathLogServer(),"Classification Rules"," C4.5 "
                                , an.getTarget() ,an.getIduracion(), an.getIcantidad(), an.getItamano());
                        /** centrar*/
                        mt.setLocationRelativeTo(mt.getParent());
                        /**Visible*/
                        mt.setVisible(true);      
                    }
                    // *** Finished ***
                    //getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }
            else if(sourceClassName.equals(MateTreeNode.class.getCanonicalName())){
                
                MateTreeNode   fn = (MateTreeNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia fptree*/
                //***********************************************************
                
                this.setPathLogServer(fn.getPathLogServer());
                System.out.println(" From Rules V: " + this.getPathLogServer());
                if(!fn.isConfigured()){
                    this.getParent().showError("MateTree must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!fn.isExecuted()){
                    this.getParent().showError("MateTree must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**EXECUTION CODE GOES HERE*/
                    /**se visibiliza el formulario*/
                    // *** Polaris *** Johana ***
                    if(fn==null){
                        this.setMsg("Error: Mate Node don't load...");
                    }else if(fn.getArbol()==null){
                        this.setMsg("Error: Desicion Tree isn't exist");
                    }else if(fn.getPathLogServer()==null || fn.getTarget()==null){
                        this.setMsg("Error: Settings Parameters don't exist");
                    }else{                        
                    //
                        createRulesCla crc= new createRulesCla(fn.getArbol(), fn.getTarget());
                        //crc.MostrarReglas();
                        MostrarTabla mt= new MostrarTabla(crc.getreglas(),fn.getPathLogServer(),"Classification Rules"," C4.5 "
                                , fn.getTarget() ,fn.getIduracion(), fn.getIcantidad(), fn.getItamano());
                        /** centrar*/
                        mt.setLocationRelativeTo(mt.getParent());
                        /**Visible*/
                        mt.setVisible(true);      
                    }
                    // *** Finished ***
                    //getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }
            
        } 
        
        ///////////////////////////////////////////////////////////////////////////
       ////////////////////////////////////////////////////////////////////////// 
        
        else{
            this.getParent().showError("First add a connection");
            /**validacion para la badera ejecucion y el color del borde*/
            this.setExecuted(false);
            setBorderColor(null);
            getParent().repaint();
        }
   }
       
    public String getPathLogServer() {
        return pathLogServer;
    }
    
    public void setPathLogServer(String pathLogServer) {
        this.pathLogServer = pathLogServer;
    }
    
    public int getTipoL() {
        return tipoL;
    }
    
    public void setTipoL(int tipoL) {
        this.tipoL = tipoL;
    }
    
    public int getTsesion() {
        return getTsesion();
    }
    
    public void setTsesion(int tSesion) {
        this.setTsesion(getTsesion());
    }        
}
