/**el archivo contiene las configuraciones correspondientes a la seleccion Weka View
*desde View*/

package workspace.Views;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import workspace.Classification.algorithms.C45Node;
import workspace.Classification.algorithms.CBANode;
import workspace.Classification.algorithms.MateTreeNode;
import workspace.Main_Set.SimpleNode;

public class WekaViewNode extends SimpleNode{
    /**se esta forzando a variable*/
    private Class<?>[] clases;
    private ArrayList<SimpleNode> nodos;
    /**las siguientes banderas establecen si el objeto ya fue configurado
    *o ejecutado*/
    public boolean configured = true;
    public boolean executed   = false;
    public String pathLogServer = "";
    public int tipoL;
    public int tSesion;
    
    public WekaViewNode(Point center) {
        /**se establece centro, dimension*/
        this.setCenter(center);
        this.setDimension(new Dimension(80,60));
        /**se valida el nodo antecesor, 2 nodos le pueden anteceder*/
        this.clases = new Class<?>[3];
        this.clases[0] = C45Node.class;
        this.clases[1] = MateTreeNode.class;
        this.clases[2] = CBANode.class;
        /**se instancia un ArrayList en nodos*/
        this.nodos = new ArrayList();
    }
    
    public Class<?>[] getValidInputClasses(){
        /**retorna los antecesores...*/
        return this.clases;
    }
    
    public ArrayList<SimpleNode> getInputNodes(){
        /**se devuelven los valores de los nodos en un array forzado a tipo
        *SimpleNode*/
        return this.nodos;
    }
    
    public Image getImage(){
        /**el icono*/
        return new ImageIcon(System.class.getResource("/media/views.png")).getImage();
    }
    
    public String getString(){
        /**retorna una cadena que lo identifica*/
        return "Weka";
    }
    
    /**si el nodo puede ser conectado, entonces se aade un nodo*/
    public void connect(SimpleNode n){
        if(this.canConnect(n)){
            this.nodos.add(n);
        }
    }
    
    /**al desconectar el nodo se elimina la linea punteada y se coloca el color
    de borde por defecto*/
    public void disconnect(SimpleNode n){
        this.nodos.remove(n);
        this.setBorderColor(null);
    }
    
    /**retorna verdadero porque puede ser ejecutado*/
    public boolean canExecute() {
        return true;
    }
    
    public void execute() {
        // *** Polaris *** Johana ***
        // *** Finished ***
        if(this.getInputNodes().size()==1){
            String sourceClassName = this.getInputNodes().get(0).getClass().getCanonicalName();
            if(sourceClassName.equals(C45Node.class.getCanonicalName())){
                
                C45Node an = (C45Node)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia c45*/
                //***********************************************************
                
                /**paso del path desde select source*/
                this.setPathLogServer(an.getPathLogServer());
                System.out.println(" From Weka V: " + this.getPathLogServer());
                /**paso del tiempo de sesion desde prepro
                *validaciones correspondientes en caso de que no este validado*/
                if(!an.isConfigured()){
                    this.getParent().showError("c45 must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!an.isExecuted()){
                    this.getParent().showError("c45 must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**se visibiliza el formulario*/
                    // *** Polaris *** Johana ***
                    //new MostrarArbol(an.getPathLogServer()," C4.5 ",an.getTarget(),an.getArbol()).setVisible(true);
                    // *** Finished ***
                    getParent().repaint();
                    
                    this.setExecuted(true);
                    /**pinta de verde si el nodo ya se ejecuto*/
                    setBorderColor(new Color(0,255,0));
                    getParent().repaint();
                }
            }else if(sourceClassName.equals(CBANode.class.getCanonicalName())){
                
                CBANode   en = (CBANode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia cba*/
                //***********************************************************
                this.setPathLogServer(en.getPathLogServer());
                System.out.println(" From Weka: " + this.getPathLogServer());
                if(!en.isConfigured()){
                    this.getParent().showError("CBA must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!en.isExecuted()){
                    this.getParent().showError("CBA must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /*EXECUTION CODE GOES HERE
                    *se visibiliza el formulario*/
                    MostrarArbol ft = new MostrarArbol(null,true);
                    ft.setVisible(true);
                    getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    *al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            } else if(sourceClassName.equals(MateTreeNode.class.getCanonicalName())){
                
                MateTreeNode   fn = (MateTreeNode)this.getInputNodes().get(0);
                //***********************************************************
                /*aqui se instancia matetree*/
                //***********************************************************
                //new MostrarArbol(fn.getPathLogServer()," Mate ",fn.getTarget(),fn.getArbol()).setVisible(true);
                
                this.setPathLogServer(fn.getPathLogServer());
                //System.out.println(" From Weka: " + this.getPathLogServer());
                if(!fn.isConfigured()){
                    this.getParent().showError("MateTree must be configured first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else if(!fn.isExecuted()){
                    this.getParent().showError("MateTree must be executed first");
                    /**validacion para la badera ejecucion y el color del borde*/
                    this.setExecuted(false);
                    setBorderColor(null);
                } else{
                    /**EXECUTION CODE GOES HERE
                    *se visibiliza el formulario*/
                    MostrarArbol ft = new MostrarArbol(null,true);
                    ft.setVisible(true);
                    getParent().repaint();
                    
                    /**cambia el valor de la bandera executed*/
                    this.setExecuted(true);
                    /**si el nodo fue configurado, el borde se torna verde*/
                    setBorderColor(new Color(0,255,0));
                    /**obtiene el contenedor y lo redibuja... si no se redibuja entonces
                    *al mostrarse graficamente presenta pixeles defectuosos*/
                    getParent().repaint();
                }
            }
        } else{
            this.getParent().showError("First add a connection");
            /**validacion para la badera ejecucion y el color del borde*/
            this.setExecuted(false);
            setBorderColor(null);
            getParent().repaint();
        }
    }
    
    public String getPathLogServer() {
        return pathLogServer;
    }
    
    public void setPathLogServer(String pathLogServer) {
        this.pathLogServer = pathLogServer;
    }
    
    public int getTipoL() {
        return tipoL;
    }
    
    public void setTipoL(int tipoL) {
        this.tipoL = tipoL;
    }
    
    public int getTsesion() {
        return getTsesion();
    }
    
    public void setTsesion(int tSesion) {
        this.setTsesion(getTsesion());
    }
}
